/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 Ant-Contrib project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Ant-Contrib project (http://sourceforge.net/projects/ant-contrib)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The name Ant-Contrib must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact
 *    ant-contrib-developers@lists.sourceforge.net.
 *
 * 5. Products derived from this software may not be called "Ant-Contrib"
 *    nor may "Ant-Contrib" appear in their names without prior written
 *    permission of the Ant-Contrib project.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE ANT-CONTRIB PROJECT OR ITS
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */
package net.sf.antcontrib.logic;

import org.apache.tools.ant.Task;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.types.Mapper;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.taskdefs.condition.Condition;
import org.apache.tools.ant.taskdefs.UpToDate;

import java.io.File;


/****************************************************************************
 * Place class description here.
 *
 * @author		inger
 *
 * @since
 *               
 ****************************************************************************/
public class OutofdateTask
        extends Task
        implements Condition
{
    private String property;
    private String value = "true";
    private Mapper mapper;
    private Path targetpaths = null;
    private Path sourcepaths = null;
    private UpToDate upToDate = new UpToDate();

    public OutofdateTask()
    {
        super();
    }

    /**
     * The property to set if any of the target files are outofdate with
     * regard to any of the source files.
     *
     * @param property the name of the property to set if Target is outofdate.
     */
    public void setProperty(String property) {
        this.property = property;
    }

    /**
     * The value to set the named property to the target files
     * are outofdate
     *
     * @param value the value to set the property
     */
    public void setValue(String value) {
        this.value = value;
    }

    /***
     * Create the <targetfiles> subelement
     * @return
     */
    public Path createTargetfiles()
    {
        if (targetpaths == null) {
            targetpaths = new Path(getProject());
        }
        return targetpaths;
    }

    /***
     * Create the <sourcefiles> subelement
     * @return
     */
    public Path createSourcefiles()
    {
        if (sourcepaths == null) {
            sourcepaths = new Path(getProject());
        }
        return sourcepaths;
    }


    /**
     * Evaluate (all) target and source file(s) to
     * see if the target(s) is/are outoutdate.
     */
    public boolean eval()
        throws BuildException
    {
        upToDate.setProject(getProject());
        upToDate.setTaskName(getTaskName());
        if (sourcepaths == null)
            throw new BuildException(
                    "You must specify a <sourcefiles> element.");

        if (targetpaths == null)
            throw new BuildException(
                    "You must specify a <targetfiles> element.");


        String spaths[] = sourcepaths.list();
        String tpaths[] = targetpaths.list();

        for (int i=0;i<spaths.length;i++)
        {
            File sourceFile = new File(spaths[i]);
            if (! sourceFile.exists())
            {
                throw new BuildException(sourceFile.getAbsolutePath()
                                         + " not found.");
            }
        }


        for (int i=0;i<tpaths.length;i++)
        {
            if (targetNeedsGen(tpaths[i], spaths))
                return true;
        }

        return false;
    }

    /***
     * Se if a given targetfile needs re-generation
     * @param targetFile
     * @param spaths
     * @return
     */
    private boolean targetNeedsGen(String targetFile, String spaths[]) {
        upToDate.setTargetFile(new File(targetFile));
        if (spaths.length == 0)
            return (! upToDate.eval());

        for (int i=0;i<spaths.length;i++)
        {
            upToDate.setSrcfile(new File(spaths[i]));
            if (! upToDate.eval())
                return true;
        }

        return false;
    }

    public void execute()
        throws BuildException
    {
        if (property == null)
            throw new BuildException(
            "property attribute is required.");

        boolean eval = eval();
        if (eval)
            this.getProject().setNewProperty(property, value);
        
    }
}
